@@ -6,24 +6,31 @@ libtmux.session
 """
 import logging
 import os
+import typing as t
 
 from . import exc, formats
 from .common import (
     EnvironmentMixin,
     TmuxMappingObject,
     TmuxRelationalObject,
+    WindowDict,
     handle_option_error,
     has_version,
     session_check_name,
 )
 from .window import Window
 
+if t.TYPE_CHECKING:
+    from .pane import Pane
+    from .server import Server
+
+
 logger = logging.getLogger(__name__)
 
 
 class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
     """
-    A :term:`tmux(1)` :term:`Session` [#]_.
+    A :term:`tmux(1)` :term:`Session` [session_manual]_.
 
     Holds :class:`Window` objects.
 
@@ -33,7 +40,7 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
 
     References
     ----------
-    .. [#] tmux session. openbsd manpage for TMUX(1).
+    .. [session_manual] tmux session. openbsd manpage for TMUX(1).
            "When tmux is started it creates a new session with a single window
            and displays it on screen..."
 
@@ -44,36 +51,40 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
        https://man.openbsd.org/tmux.1#DESCRIPTION. Accessed April 1st, 2018.
     """
 
-    #: unique child ID key for :class:`~libtmux.common.TmuxRelationalObject`
-    child_id_attribute = 'window_id'
-    #: namespace used :class:`~libtmux.common.TmuxMappingObject`
-    formatter_prefix = 'session_'
+    child_id_attribute = "window_id"
+    """Unique child ID key for :class:`~libtmux.common.TmuxRelationalObject`"""
+    formatter_prefix = "session_"
+    """Namespace used for :class:`~libtmux.common.TmuxMappingObject`"""
+    server: "Server"
+    """:class:`libtmux.server.Server` session is linked to"""
 
-    def __init__(self, server=None, **kwargs):
+    def __init__(self, server: "Server", **kwargs):
         EnvironmentMixin.__init__(self)
         self.server = server
 
-        if 'session_id' not in kwargs:
-            raise ValueError('Session requires a `session_id`')
-        self._session_id = kwargs['session_id']
+        if "session_id" not in kwargs:
+            raise ValueError("Session requires a `session_id`")
+        self._session_id = kwargs["session_id"]
         self.server._update_windows()
 
     @property
     def _info(self):
 
-        attrs = {'session_id': str(self._session_id)}
+        attrs = {"session_id": str(self._session_id)}
 
-        def by(val):
-            for key, value in attrs.items():
+        def by(val) -> bool:
+            for key in attrs.keys():
                 try:
                     if attrs[key] != val[key]:
                         return False
                 except KeyError:
                     return False
-                return True
+            return True
 
+        # TODO add type hint
+        target_sessions = list(filter(by, self.server._sessions))
         try:
-            return list(filter(by, self.server._sessions))[0]
+            return target_sessions[0]
         except IndexError as e:
             logger.error(e)
 
@@ -92,17 +103,17 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
             Renamed from ``.tmux`` to ``.cmd``.
         """
         # if -t is not set in any arg yet
-        if not any('-t' in str(x) for x in args):
+        if not any("-t" in str(x) for x in args):
             # insert -t immediately after 1st arg, as per tmux format
             new_args = [args[0]]
-            new_args += ['-t', self.id]
+            new_args += ["-t", self.id]
             new_args += args[1:]
             args = new_args
         return self.server.cmd(*args, **kwargs)
 
     def attach_session(self):
         """Return ``$ tmux attach-session`` aka alias: ``$ tmux attach``."""
-        proc = self.cmd('attach-session', '-t%s' % self.id)
+        proc = self.cmd("attach-session", "-t%s" % self.id)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
@@ -110,7 +121,7 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
     def kill_session(self):
         """``$ tmux kill-session``."""
 
-        proc = self.cmd('kill-session', '-t%s' % self.id)
+        proc = self.cmd("kill-session", "-t%s" % self.id)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
@@ -124,12 +135,12 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
 
         :exc:`exc.LibTmuxException`
         """
-        proc = self.cmd('switch-client', '-t%s' % self.id)
+        proc = self.cmd("switch-client", "-t%s" % self.id)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
 
-    def rename_session(self, new_name):
+    def rename_session(self, new_name: str) -> "Session":
         """
         Rename session and return new :class:`Session` object.
 
@@ -148,10 +159,10 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
         """
         session_check_name(new_name)
 
-        proc = self.cmd('rename-session', new_name)
+        proc = self.cmd("rename-session", new_name)
 
         if proc.stderr:
-            if has_version('2.7') and 'no current client' in proc.stderr:
+            if has_version("2.7") and "no current client" in proc.stderr:
                 """tmux 2.7 raises "no current client" warning on BSD systems.
 
                 Should be fixed next release:
@@ -159,7 +170,6 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
                 - https://www.mail-archive.com/tech@openbsd.org/msg45186.html
                 - https://marc.info/?l=openbsd-cvs&m=152183263526828&w=2
                 """
-                pass
             else:
                 raise exc.LibTmuxException(proc.stderr)
 
@@ -170,9 +180,9 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
         window_name=None,
         start_directory=None,
         attach=True,
-        window_index='',
+        window_index="",
         window_shell=None,
-    ):
+    ) -> Window:
         """
         Return :class:`Window` from ``$ tmux new-window``.
 
@@ -202,45 +212,47 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
         -------
         :class:`Window`
         """
-        wformats = ['session_name', 'session_id'] + formats.WINDOW_FORMATS
-        tmux_formats = ['#{%s}' % f for f in wformats]
+        wformats = ["session_name", "session_id"] + formats.WINDOW_FORMATS
+        tmux_formats = ["#{%s}" % f for f in wformats]
 
-        window_args = tuple()
+        window_args: t.Tuple = tuple()
 
         if not attach:
-            window_args += ('-d',)
+            window_args += ("-d",)
 
-        window_args += ('-P',)
+        window_args += ("-P",)
 
         if start_directory:
             # as of 2014-02-08 tmux 1.9-dev doesn't expand ~ in new-window -c.
             start_directory = os.path.expanduser(start_directory)
-            window_args += ('-c%s' % start_directory,)
+            window_args += ("-c%s" % start_directory,)
 
-        window_args += ('-F"%s"' % '\t'.join(tmux_formats),)  # output
+        window_args += (
+            '-F"%s"' % formats.FORMAT_SEPARATOR.join(tmux_formats),
+        )  # output
         if window_name:
-            window_args += ('-n%s' % window_name,)
+            window_args += ("-n%s" % window_name,)
 
         window_args += (
             # empty string for window_index will use the first one available
-            '-t%s:%s'
+            "-t%s:%s"
             % (self.id, window_index),
         )
 
         if window_shell:
             window_args += (window_shell,)
 
-        proc = self.cmd('new-window', *window_args)
+        proc = self.cmd("new-window", *window_args)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
 
         window = proc.stdout[0]
 
-        window = dict(zip(wformats, window.split('\t')))
+        window = dict(zip(wformats, window.split(formats.FORMAT_SEPARATOR)))
 
         # clear up empty dict
-        window = dict((k, v) for k, v in window.items() if v)
+        window = {k: v for k, v in window.items() if v}
         window = Window(session=self, **window)
 
         self.server._update_windows()
@@ -261,43 +273,41 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
 
         if target_window:
             if isinstance(target_window, int):
-                target = '-t%s:%d' % (self.name, target_window)
+                target = "-t%s:%d" % (self.name, target_window)
             else:
-                target = '-t%s' % target_window
+                target = "-t%s" % target_window
 
-        proc = self.cmd('kill-window', target)
+        proc = self.cmd("kill-window", target)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
 
         self.server._update_windows()
 
-    def _list_windows(self):
+    def _list_windows(self) -> t.List[WindowDict]:
         windows = self.server._update_windows()._windows
 
-        windows = [w for w in windows if w['session_id'] == self.id]
-
-        return windows
+        return [w for w in windows if w["session_id"] == self.id]
 
     @property
-    def _windows(self):
+    def _windows(self) -> t.List[WindowDict]:
         """Property / alias to return :meth:`Session._list_windows`."""
 
         return self._list_windows()
 
-    def list_windows(self):
+    def list_windows(self) -> t.List[Window]:
         """Return a list of :class:`Window` from the ``tmux(1)`` session.
 
         Returns
         -------
         :class:`Window`
         """
-        windows = [w for w in self._windows if w['session_id'] == self._session_id]
+        windows = [w for w in self._windows if w["session_id"] == self._session_id]
 
         return [Window(session=self, **window) for window in windows]
 
     @property
-    def windows(self):
+    def windows(self) -> t.List[Window]:
         """Property / alias to return :meth:`Session.list_windows`."""
         return self.list_windows()
 
@@ -305,7 +315,7 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
     children = windows
 
     @property
-    def attached_window(self):
+    def attached_window(self) -> Window:
         """
         Return active :class:`Window` object.
 
@@ -315,24 +325,21 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
         """
         active_windows = []
         for window in self._windows:
-            if 'window_active' in window:
-                # for now window_active is a unicode
-                if window.get('window_active') == '1':
-                    active_windows.append(Window(session=self, **window))
-                else:
-                    continue
+            # for now window_active is a unicode
+            if "window_active" in window and window.get("window_active") == "1":
+                active_windows.append(Window(session=self, **window))
 
-        if len(active_windows) == int(1):
+        if len(active_windows) == 1:
             return active_windows[0]
         else:
             raise exc.LibTmuxException(
-                'multiple active windows found. %s' % active_windows
+                "multiple active windows found. %s" % active_windows
             )
 
-        if len(self._windows) == int(0):
-            raise exc.LibTmuxException('No Windows')
+        if len(self._windows) == 0:
+            raise exc.LibTmuxException("No Windows")
 
-    def select_window(self, target_window):
+    def select_window(self, target_window: str) -> Window:
         """
         Return :class:`Window` selected via ``$ tmux select-window``.
 
@@ -353,9 +360,12 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
             assure ``-l``, ``-n``, ``-p`` work.
         """
 
-        target = '-t%s' % target_window
+        # Note that we also provide the session ID here, since cmd()
+        # will not automatically add it as there is already a '-t'
+        # argument provided.
+        target = f"-t{self._session_id}:{target_window}"
 
-        proc = self.cmd('select-window', target)
+        proc = self.cmd("select-window", target)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
@@ -363,7 +373,7 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
         return self.attached_window
 
     @property
-    def attached_pane(self):
+    def attached_pane(self) -> t.Optional["Pane"]:
         """Return active :class:`Pane` object."""
 
         return self.attached_window.attached_pane
@@ -395,18 +405,18 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
         """
 
         if isinstance(value, bool) and value:
-            value = 'on'
+            value = "on"
         elif isinstance(value, bool) and not value:
-            value = 'off'
+            value = "off"
 
         tmux_args = tuple()
 
         if _global:
-            tmux_args += ('-g',)
+            tmux_args += ("-g",)
 
         tmux_args += (option, value)
 
-        proc = self.cmd('set-option', *tmux_args)
+        proc = self.cmd("set-option", *tmux_args)
 
         if isinstance(proc.stderr, list) and len(proc.stderr):
             handle_option_error(proc.stderr[0])
@@ -438,15 +448,15 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
         tmux_args = tuple()
 
         if _global:
-            tmux_args += ('-g',)
+            tmux_args += ("-g",)
 
         if option:
             return self.show_option(option, _global=_global)
         else:
-            tmux_args += ('show-options',)
+            tmux_args += ("show-options",)
             session_options = self.cmd(*tmux_args).stdout
 
-        session_options = [tuple(item.split(' ')) for item in session_options]
+        session_options = [tuple(item.split(" ")) for item in session_options]
 
         session_options = dict(session_options)
 
@@ -486,11 +496,11 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
         tmux_args = tuple()
 
         if _global:
-            tmux_args += ('-g',)
+            tmux_args += ("-g",)
 
         tmux_args += (option,)
 
-        cmd = self.cmd('show-options', *tmux_args)
+        cmd = self.cmd("show-options", *tmux_args)
 
         if isinstance(cmd.stderr, list) and len(cmd.stderr):
             handle_option_error(cmd.stderr[0])
@@ -498,7 +508,7 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
         if not len(cmd.stdout):
             return None
 
-        option = [item.split(' ') for item in cmd.stdout][0]
+        option = [item.split(" ") for item in cmd.stdout][0]
 
         if option[1].isdigit():
             option = (option[0], int(option[1]))
@@ -506,4 +516,4 @@ class Session(TmuxMappingObject, TmuxRelationalObject, EnvironmentMixin):
         return option[1]
 
     def __repr__(self):
-        return "%s(%s %s)" % (self.__class__.__name__, self.id, self.name)
+        return f"{self.__class__.__name__}({self.id} {self.name})"
